import os
import boto3
import openai
from paperqa import Docs
from urllib.parse import urlparse
from langchain.chat_models import ChatAnthropic, ChatOpenAI

def initialize_llm(documents, temperature=0.7,
                   openai_api_key=os.environ['OPENAI_API_KEY'], 
                   anthropic_api_keyos.environ['ANTHROPIC_API_KEY']):
    # Initialize the language models
    gpt4 = ChatOpenAI(model='gpt-4', temperature=temperature, openai_api_key=openai_api_key)
    claude = ChatAnthropic(model="claude-2", temperature=temperature, anthropic_api_key=anthropic_api_key)
    llm = Docs(llm=gpt4, summary_llm=claude)

    # Add documents to Paper QA
    for path in documents:
        parsed_url = urlparse(path)
        scheme = parsed_url.scheme
        ext = os.path.splitext(path)[-1].lower()

        if scheme in ['http', 'https']:
            # If it's a web URL, use add_url
            llm.add_url(path)
        elif scheme == 's3':
            # If it's an S3 URL, download the file first
            s3 = boto3.client('s3')
            bucket = parsed_url.netloc
            key = parsed_url.path.lstrip('/')
            local_path = f"/tmp/{os.path.basename(key)}"
            s3.download_file(bucket, key, local_path)

            # Then add it based on its extension
            if ext == '.pdf':
                llm.add(local_path)
            else:
                with open(local_path, 'r') as f:
                    text_content = f.read()
                    llm.add_file(f, text_content)
        else:
            # If it's a local file, add it based on its extension
            if ext == '.pdf':
                llm.add(path)
            else:
                with open(path, 'r') as f:
                    text_content = f.read()
                    llm.add_file(f, text_content)
    
    return llm



import time
import requests
from urllib.parse import urljoin
from selenium import webdriver

class GoogleUtilWrapper:
    def __init__(self, p2_url=None, gdt_url=None, pdest=None, gdest=None):
        self.p2_url = p2_url
        self.gdt_url = gdt_url
        self.pdest = pdest
        self.gdest = gdest

    def get_data(self, ep='', fields=None):
        payload = {}
        payload["p2"] = self.p2_url
        if fields: payload["fields"] = fields
        response = requests.get(self.gdt_url + ep, json=payload)
        return response.json()

    def post_doc_data(self, fields=[], data=None):
        payload = {}
        payload["p2"] = self.p2_url
        payload["fields"] = fields
        payload["destination"] = self.pdest
        if data: payload["data"] = data
        response = requests.post(self.gdt_url, json=payload)

    def post_sheet_data(self, fields=[], sheet=None, column=None, data=None):
        payload = {}
        payload["p2"] = self.p2_url
        payload["fields"] = fields
        payload["destination"] = self.gdest
        if data: payload["data"] = data
        if sheet: payload["sheet"] = sheet
        if column: payload["source"] = column
        response = requests.post(self.gdt_url, json=payload)
        
    def get_recording_data(self, interview_type=None, max_attempts=20, sleep_interval=1, return_url=True):
        url = self.get_data(interview_type)

        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument("--headless")
        chrome_options.add_argument("--no-sandbox")
        chrome_options.add_argument("--disable-dev-shm-usage")
        
        driver = webdriver.Chrome(options=chrome_options)
        
        # Browse to the target website
        driver.get(url)
        
        # Inject JavaScript to monitor XHR responses
        script = '''
        (function() {
            var open = window.XMLHttpRequest.prototype.open;
            window.responses = [];
            window.XMLHttpRequest.prototype.open = function(method, url) {
                this.addEventListener('load', function() {
                    if(this.responseText.startsWith('WEBVTT')) {
                        window.responses.push({content: this.responseText, url: url});
                    }
                });
                open.apply(this, arguments);
            };
        })();
        '''
        driver.execute_script(script)
        
        # Poll for captured responses
        attempts = 0
        webvtt_content = None
        webvtt_url = None
        while attempts < max_attempts:
            webvtt_responses = driver.execute_script('return window.responses')
            if webvtt_responses:
                webvtt_content = webvtt_responses[0]['content']
                webvtt_url = webvtt_responses[0]['url']
                break
            time.sleep(sleep_interval)
            attempts += 1
        
        driver.quit()
        
        if return_url:
            # Append the TLD from the request to the resultant URL
            full_url = urljoin(url, webvtt_url)
            return full_url
        else:
            return webvtt_content 
